package xyz.tbvns;

import com.badlogic.gdx.ApplicationAdapter;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.graphics.Camera;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.PerspectiveCamera;
import com.badlogic.gdx.graphics.g3d.*;
import com.badlogic.gdx.graphics.g3d.attributes.ColorAttribute;
import com.badlogic.gdx.graphics.g3d.shaders.DefaultShader;
import com.badlogic.gdx.graphics.g3d.utils.ModelBuilder;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.physics.bullet.Bullet;
import com.badlogic.gdx.utils.viewport.Viewport;
import lombok.Getter;
import org.lwjgl.opengl.GL11;
import xyz.tbvns.Editor.ModelsManager;
import xyz.tbvns.Inputs.Shortcut;
import xyz.tbvns.Inputs.inputManager;
import xyz.tbvns.LWJGL3.CustomViewport;
import xyz.tbvns.Managers.TexturesManager;
import xyz.tbvns.Models.GDModel;
import xyz.tbvns.Models.Shaders;
import xyz.tbvns.Physics.BulletManager;
import xyz.tbvns.ui.UIManager;
import xyz.tbvns.ui.Ui;
import xyz.tbvns.ui.Windows.LevelSelector;

import static org.lwjgl.opengl.GL11.GL_CULL_FACE;

/** {@link com.badlogic.gdx.ApplicationListener} implementation shared by all platforms. */
public class Main extends ApplicationAdapter {
    private ModelBatch batch;
    @Getter static private Camera camera;
    private Model worldGrid;
    private Model levelGrid;
    private ModelInstance worldGridInstance;
    private ModelInstance levelGridInstance;
    private InputProcessor cameraInputController;
    private Viewport viewport;
    public static ModelCache modelCache;
    public static Model selectedModel;

    @Override
    public void create() {
        Setup.run();
        Bullet.init();
        BulletManager.innit();
        Ui.initImGui();
        UIManager.loadUi();
        UIManager.getActiveUI().put(LevelSelector.class, true);
        TexturesManager.idFileMap.size();
        batch = new ModelBatch();
        camera = new PerspectiveCamera(80, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
        camera.far = 1000;
        camera.near = 0.01f;
        viewport = new CustomViewport(camera);

        Gdx.gl.glEnable(GL11.GL_BLEND);
        Gdx.gl.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
        Gdx.gl.glEnable(GL20.GL_DEPTH_TEST);

        ModelBuilder builder = new ModelBuilder();

        //TODO: Make the grids configurable
        worldGrid = builder.createLineGrid(100, 100, 1, 1, new Material(ColorAttribute.createDiffuse(0.2f, 0.2f, 0.2f, 1)), 1);
        levelGrid = builder.createLineGrid(35, 35, 1, 1, new Material(ColorAttribute.createDiffuse(0.3f, 0.3f, 0.3f, 1)), 1);
        worldGridInstance = new ModelInstance(worldGrid);
        levelGridInstance = new ModelInstance(levelGrid);
        levelGridInstance.transform.setToRotation(new Vector3(1, 0, 0), 90);
        levelGridInstance.transform.setTranslation(new Vector3(0, 50, 0));

        cameraInputController = new inputManager(camera);
        Gdx.input.setInputProcessor(cameraInputController);
    }

    @Override
    public void render() {
        BulletManager.update();

        Gdx.gl.glViewport(0, 0, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
        Gdx.gl.glClearColor(0.1f, 0.1f, 0.1f, 1);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT | GL20.GL_DEPTH_BUFFER_BIT | (Gdx.graphics.getBufferFormat().coverageSampling ? GL20.GL_COVERAGE_BUFFER_BIT_NV : 0));
        Gdx.gl.glDisable(GL_CULL_FACE);

        viewport.setScreenPosition(200, 200);
        viewport.update(Gdx.graphics.getWidth() - 200, Gdx.graphics.getHeight(), false);

        //TODO: move grid rendering elsewhere
        Vector3 gridPos = new Vector3(camera.position);
        gridPos.x = Math.round(gridPos.x);
        gridPos.y = -0.01f;
        gridPos.z = Math.round(gridPos.z);

        worldGridInstance.transform.setTranslation(gridPos);

        gridPos = new Vector3(camera.position);
        gridPos.x = Math.round(gridPos.x);
        gridPos.y = Math.round(gridPos.y);
        gridPos.z = 0;

        levelGridInstance.transform.setTranslation(gridPos);

        batch.begin(camera);
        if (modelCache != null) {
            batch.render(modelCache);
        }
        batch.render(worldGridInstance);
        batch.render(levelGridInstance);
        for (ModelInstance model : ModelsManager.getLoadedModels()) {
            if (ModelsManager.getSelectedModelInstance() == model) {
                Shaders.renderOutline(model);
            }
        }
        for (ModelInstance model : ModelsManager.getLoadedModels()) {
            batch.render(model);
        }
        batch.end();

        Ui.render();
        Shortcut.check();
    }

    @Override
    public void dispose() {
        batch.dispose();
        worldGrid.dispose();
        levelGrid.dispose();
        Ui.dispose();
    }

    @Override
    public void resize(int width, int height) {

    }
}
