package xyz.tbvns.Models;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Mesh;
import com.badlogic.gdx.graphics.g3d.ModelInstance;
import com.badlogic.gdx.graphics.g3d.model.Node;
import com.badlogic.gdx.graphics.g3d.model.NodePart;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.utils.GdxRuntimeException;
import org.apache.commons.codec.Resources;
import xyz.tbvns.Main;

import java.io.IOException;

public class Shaders {
    public static final ShaderProgram outlineShader;

    static {
        try {
            outlineShader = new ShaderProgram(
                    new String(Resources.getInputStream("shaders/outline.vs.glsl").readAllBytes()),
                    new String(Resources.getInputStream("shaders/outline.fs.glsl").readAllBytes())
            );
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        if(!outlineShader.isCompiled())
            throw new GdxRuntimeException("Outline Shader failed: " + outlineShader.getLog());
    }

    public static void renderOutline(ModelInstance instance) {
        // Enable face culling (front-face culling to show only backfaces)
        Gdx.gl.glEnable(GL20.GL_CULL_FACE);
        Gdx.gl.glCullFace(GL20.GL_FRONT);  // Cull front faces to render only the back

        // Bind the outline shader
        outlineShader.bind();

        // Set shader uniforms
        outlineShader.setUniformMatrix("u_projTrans", Main.getCamera().combined);
        outlineShader.setUniformf("u_outlineThickness", 0.02f);  // Adjust this for outline thickness
        outlineShader.setUniformf("u_outlineColor", new Color(1f, 0.5f, 0, 0));  // Outline color

        // Render the modelInstance with the outline shader
        for (Node node : instance.nodes) {
            if (node.parts.size > 0) {
                for (NodePart part : node.parts) {
                    Mesh mesh = part.meshPart.mesh;
                    mesh.bind(outlineShader);
                    mesh.render(outlineShader, part.meshPart.primitiveType);
                    mesh.unbind(outlineShader);
                }
            }
        }

        // Disable face culling after rendering
        Gdx.gl.glDisable(GL20.GL_CULL_FACE);
    }
}
